import React, { useContext, useEffect } from 'react'
import PropTypes from 'prop-types'
import { GraphContext, isVisible } from '../utils/index.js'
import ColorTracker from 'canvas-color-tracker'
import throttle from 'lodash.throttle'
import * as d3 from 'd3-selection'



const Links = ({
    stroke = '#ccc',
    width = 1.5,
    onMouseOver,
    onMouseOut,
    onMouseMove
}) => {
    const {canvas, simulation, links, bounds} = useContext(GraphContext)
    const {shadowCanvas} = useContext(GraphContext)

    useEffect(() => {
        const container = d3.select(canvas.current)
        const ctx = canvas.current.getContext('2d')
        const shadowCtx = shadowCanvas.getContext('2d')
        const colorTracker = new ColorTracker()
        let hovered = null

        links.forEach(d => {
            d.shadowColor = colorTracker.register(d)
        })

        simulation.on('tick.links', () => {
            const { transform } = simulation
            const contexts = [ctx, shadowCtx]

            contexts.forEach((ctx, i) => {
                ctx.save()
    
                if (transform) {
                    ctx.translate(transform.x, transform.y)
                    ctx.scale(transform.k, transform.k)
                }
    
                links.forEach(d => {
                    if (
                        isVisible(d.source, transform, bounds) ||
                        isVisible(d.target, transform, bounds)
                    ) {
                        ctx.beginPath()
                        ctx.lineWidth = i === 1 ? 10 : width
                        ctx.strokeStyle = i === 1 ? d.shadowColor : stroke
                        ctx.moveTo(d.source.x, d.source.y)
                        ctx.lineTo(d.target.x, d.target.y)
                        ctx.stroke()
                    }
                })
    
                ctx.restore()
            })
        })

        const throttled = throttle((x, y) => {
            if (x > 0 && y > 0 && simulation.alpha() < 0.05) {
                const { data } = shadowCtx.getImageData(0, 0, bounds.width, bounds.height)
                const idx = (y * bounds.width + x) * 4
                const parts = Array.prototype.slice.call(data,idx,idx + 4)
                const link = colorTracker.lookup(parts)

                if (!hovered && link && onMouseOver) onMouseOver(link) 
                if (hovered && !link && onMouseOut) onMouseOut(link)
                if (onMouseMove) onMouseMove(link, { x, y })

                hovered = link
            }
        }, 50)

        container.on('mousemove.links', () => {
            const { offsetX: x, offsetY: y } = d3.event

            throttled(x, y)
        })
    })

    return null
}

Links.propTypes = {
    attributes: PropTypes.object,
    style: PropTypes.object,
    onTick: PropTypes.func
}

export default Links