import { createContext } from 'react'

/**
 * Exposes top-level graph data, simulation, and more.
 * 
 * @type {object}
 */
export const GraphContext = createContext({
    simulation: null,
    nodes: null,
    links: null
})

/**
 * Account for canvas offset in the given x and y coordinates.
 *
 * @param  {number} x 	   - X coordinate
 * @param  {number} y 	   - Y coordinate
 * @param  {object} canvas - A canvas element
 * @return {object} 	   - The offset `x` and `y` coordinates
 */
export const getRelativePosition = (x, y, canvas) => {
    const {top, left} = canvas.getBoundingClientRect()

    return {
        x: x - left,
        y: y - top
    }
}

/**
 * Determine whether the given `node` is visible within the `bounds`.
 *
 * @param  {object}  node      - A valid graph node (containing `x` / `y` coordinates)
 * @param  {object}  transform - A d3 transform instance
 * @param  {object}  bounds    - The `width` and `height` of the graph
 * @return {boolean}           - Indicates whether the node is visible
 */
export const isVisible = (node, transform, bounds) => {
    const {width, height} = bounds
    const [x, y] = transform.apply([node.x, node.y])

    return  x > 0 && x < width && y > 0 && y < height
}

/**
 * Merge incoming nodes with existing ones to provide a more seamless
 * transition as the incoming data changes. Without this, nodes will always
 * start at the center of graph which doesn't make sense if they were already
 * rendered. We do this combination based on each node's `id` field.
 * 
 * @param  {array} newNodes - ...
 * @param  {array} oldNodes - ...
 * @return {array}          - A copy of `newNodes` with 
 */
export const mergeNodes = (newNodes, oldNodes) => {
    return newNodes.map(node => {
        const previous = oldNodes.find(obj => obj.id === node.id)

        if (!previous) return node
        else return {
            ...previous,
            ...node
        }
    })
}

/**
 * Flatten a nested object.
 *
 * @see https://stackoverflow.com/questions/19098797/fastest-way-to-flatten-un-flatten-nested-json-objects
 * @param  {object} data - ...
 * @return {object}      - ...
 */
export const flatten = data => {
    var result = {}

    function recurse (cur, prop) {
        if (Object(cur) !== cur) {
            result[prop] = cur

        } else if (Array.isArray(cur)) {
            for (var i = 0, l = cur.length; i < l; i++)
                recurse(cur[i], prop + '[' + i + ']')
            if (l == 0)
                result[prop] = []

        } else {
            let isEmpty = true

            for (let p in cur) {
                isEmpty = false
                recurse(cur[p], prop ? prop + '.' + p : p)
            }

            if (isEmpty && prop)
                result[prop] = {}
        }
    }

    recurse(data, '')

    return result
}

/**
 * Dedupe and consolidate keys in a `flatten`-ed object.
 *
 * @param  {object} flat - ...
 * @return {array}       - ...
 */
export const consolidate = flat => {
    const entries = Object.entries(flat)
    const cleaned = entries.map(arr => JSON.stringify([
        arr[0].replace(/attributes\.|\[.+?\]/ig, ''),
        arr[1]
    ]))

    return Array.from(new Set(cleaned)).map(JSON.parse)
}