import { generateLinks } from './links.js'

describe('the `generateLinks` utility', () => {
    test('ignores null and undefined', () => {
        const nodes = [
            { id: 1, name: 'Jane', age: null },
            { id: 2, name: 'Frank', age: null },
            { id: 3, name: 'Jane', age: 29 }
        ]

        expect(generateLinks(nodes, nodes)).toEqual([
            { source: 1, target: 3, match: { name: 'Jane' }}
        ])
    })

    test('handles strings, booleans, and numbers', () => {
        const nodes = [
            { id: 1, name: 'Jane', age: 24 },
            { id: 2, name: 'Frank', age: 24 },
            { id: 3, name: 'Jane', age: 29 }
        ]

        expect(generateLinks(nodes, nodes)).toEqual([
            { source: 1, target: 2, match: { age: 24 }},
            { source: 1, target: 3, match: { name: 'Jane' }}
        ])
    })

    test('handles nested objects', () => {
        const nodes = [
            { id: 1, attributes: { name: 'Jane', age: 24 }},
            { id: 2, attributes: { name: 'Frank', age: 24 }},
            { id: 3, attributes: { name: 'Jane', age: 29 }}
        ]

        expect(generateLinks(nodes, nodes)).toEqual([
            { source: 1, target: 2, match: {
                attributes: { age: 24 }
            }},
            { source: 1, target: 3, match: {
                attributes: { name: 'Jane' }
            }}
        ])
    })

    test('handles nested arrays', () => {
        const nodes = [
            { id: 1, list: [ 'foo', 'bar', 'baz' ] },
            { id: 2, list: [ 'bar', 'baz', 'qux' ] },
            { id: 3, list: [ 'qux', 'quxx' ] }
        ]

        expect(generateLinks(nodes, nodes)).toEqual([
            { source: 1, target: 2, match: { list: [ 'bar', 'baz' ]}},
            { source: 2, target: 3, match: { list: [ 'qux' ]}}
        ])
    })

    test('handles nested collections', () => {
        const nodes = [
            { id: 1, list: [
                { title: 'doctor', code: 224 },
                { title: 'nurse', code: 221 },
                { title: 'student', code: 122 }
            ]},
            { id: 2, list: [
                { title: 'student', code: 122 },
                { title: 'teacher', code: 515 }
            ]}
        ]

        expect(generateLinks(nodes, nodes)).toEqual([
            { source: 1, target: 2, match: {
                list: [
                    { title: 'student', code: 122 }
                ]
            }}
        ])
    })

    test('warns about multi-dimensional arrays', () => {
        const nodes = [
            { id: 1, list: [[], []]},
            { id: 2, list: [[]]}
        ]

        global.console = { warn: jest.fn() }
        generateLinks(nodes, nodes)
        expect(console.warn).toBeCalled()
    })

    test('warns about mixed arrays', () => {
        const nodes = [
            { id: 1, list: [true, []]},
            { id: 2, list: [{}, false]}
        ]

        global.console = { warn: jest.fn() }
        generateLinks(nodes, nodes)
        expect(console.warn).toBeCalled()
    })

    test('supports exclusion of fields', () => {
        const options = { ignored: [ 'name' ] }
        const nodes = [
            { id: 1, name: 'Jane', age: 24 },
            { id: 2, name: 'Frank', age: 24 },
            { id: 3, name: 'Jane', age: 29 }
        ]

        expect(generateLinks(nodes, nodes, options)).toEqual([
            { source: 1, target: 2, match: { age: 24 }}
        ])
    })
})